/*!
 * api-render-vue v0.0.1-alpha.16
 * An API data parsing tool.
 * © 2024-2024 @jl15988(https://github.com/jl15988)
 * MIT License.
 */
import { ref, defineComponent } from 'vue';

var apiRenderVue$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get ApiRenderTool () { return ApiRenderUtil; },
    get ApiRenderUtils () { return ApiRenderUtils; },
    get default () { return apiRenderVue; },
    get defineApiRenderOptions () { return defineApiRenderOptions; },
    get getApiRenderCache () { return getApiRenderCache; },
    get getApiRenderConfig () { return getApiRenderConfig; },
    get reloadApiRenderCache () { return reloadApiRenderCache; },
    get setApiRenderCache () { return setApiRenderCache; },
    get setApiRenderConfig () { return setApiRenderConfig; }
});

const apiRenderConfig = {
    openCache: true,
    cacheTimeout: 1000 * 60 * 5,
    openClearCache: true,
    clearCacheSpace: 1000 * 60,
    defaultLabelKey: 'label',
    defaultValueKey: 'value',
    defaultTreeLabelKey: 'label',
    defaultTreeValueKey: 'value',
    defaultTreeIdKey: 'id',
    defaultTreeChildrenKey: 'children',
    defaultTreeParentIdKey: 'parentId',
    defaultOptionsValueKey: 'value',
    defaultOptionsLabelKey: 'label',
};
function setApiRenderConfig(config) {
    Object.assign(config, apiRenderConfig);
}
function getApiRenderConfig() {
    return Object.assign({}, apiRenderConfig);
}

/*
api 渲染请求缓存

如果提供有名函数，则直接将函数作为 key 来缓存数据，这样 key 的引用会指向函数

如果提供匿名函数，则通过 Map 实现，又分两种情况，提供 apiKey 或者不提供
1. 有 apiKey 时，则把 apiKey 作为 key 来缓存数据
2. 没有时，则将 api 通过 toString 函数字符串化作为缓存 key，所以这种方式是不太安全的，因为匿名函数可能会重复（但大多数情况不会）

---自动回收实现
缓存数据数据时赋值缓存时间，当获取数据时相同 key 的数据会先判断超时，如果超时则重新获取，否则直接返回
而其他数据则通过异步遍历判断超时，超时则删除，且通过记录上次清理缓存时间来防止一段时间内多次清理
 */
// api 渲染请求函数类型
// 缓存
// const apiRenderCache = new WeakMap<ApiRenderApiType, ApiRenderValueType>()
const apiRenderMap = new Map();
// 请求缓存
const requestCache = new Map();
// 上次清理缓存时间戳
let clearCacheTime = 0;
// 遍历清理缓存
function clearTimeoutCache() {
    if (!apiRenderConfig.openClearCache)
        return Promise.resolve(true);
    if (clearCacheTime + apiRenderConfig.clearCacheSpace > Date.now())
        return Promise.resolve(true);
    clearCacheTime = Date.now();
    return new Promise((resolve) => {
        for (const [key, value] of apiRenderMap.entries()) {
            if (value.timeout <= Date.now()) {
                // 如果超时，则移除该键值对
                apiRenderMap.delete(key);
            }
        }
        resolve(true);
    });
}
// 请求，通过请求缓存防止同一时间内发生多次请求
async function apiRenderRequest(api, key) {
    const requestKey = getApiCacheKey(api, key);
    let requestCacheItem = requestCache.get(requestKey);
    if (!requestCacheItem) {
        requestCache.set(requestKey, api());
        requestCacheItem = requestCache.get(requestKey);
    }
    const res = await requestCacheItem;
    requestCache.delete(requestKey);
    return res;
}
// 获取 api 缓存 key
function getApiCacheKey(api, key) {
    if (key) {
        // 如果指定 key 则直接使用 key
        return key;
    }
    else if (!api) {
        throw new Error('api and key must have one of them.');
    }
    if (!api.name) {
        // 匿名函数使用函数签名
        return api.toString();
    }
    else {
        // 具名函数使用引用
        return api;
    }
}
// 通过缓存中的缓存 key 获取 apiKey，用于缓存的重新加载
function getApiKeyByCacheKey(cacheKey) {
    if (typeof cacheKey === 'string' || typeof cacheKey === 'number') {
        return cacheKey;
    }
    return undefined;
}
/**
 * 赋值缓存
 * @param api api 请求函数，也用于缓存重载
 * @param value 缓存的值
 * @param key api 的 key 值，可用来声明 api 缓存的key，api 请求的数据会自动缓存，所以 api 函数尽量使用有名函数，或者如果使用匿名函数，在不保证匿名函数会字符会重复的前提下，请提供该字段
 */
function setApiRenderCache(value, api, key) {
    if (!api && !key) {
        throw new Error('api and key must have one of them.');
    }
    const cacheValue = {
        value: value,
        api: api || (() => Promise.resolve(undefined)),
        timestamp: Date.now(),
        timeout: Date.now() + apiRenderConfig.cacheTimeout,
    };
    apiRenderMap.set(getApiCacheKey(api, key), cacheValue);
}
/**
 * 获取 api 请求数据，优先取缓存数据，缓存不存在或缓存超时会重新请求获取
 * @param api api 请求函数
 * @param key api 的 key 值，可用来声明 api 缓存的 key，api 请求的数据会自动缓存，所以 api 函数尽量使用有名函数，或者如果使用匿名函数，在不保证匿名函数会字符会重复的前提下，请提供该字段
 */
async function getApiRenderCache(api, key) {
    // 没有 api 则返回没有
    if (!api)
        return Promise.resolve(undefined);
    // 获取缓存的数据
    const cache = apiRenderMap.get(getApiCacheKey(api, key));
    if (!apiRenderConfig.openCache || !apiRenderConfig.cacheTimeout || apiRenderConfig.cacheTimeout <= 0) {
        // 没有开启缓存或没有超时时间都走请求
        return (await apiRenderRequest(api, key));
    }
    if (!cache || (cache && apiRenderConfig.openClearCache && cache.timeout && cache.timeout <= Date.now())) {
        // 没有缓存数据重新请求
        const res = await apiRenderRequest(api, key);
        // 缓存
        setApiRenderCache(res, api, key);
        // 清理
        clearTimeoutCache();
        return res;
    }
    // 清理
    clearTimeoutCache();
    return Promise.resolve(cache.value);
}
// 通过缓存 key 和缓存内容重新加载缓存
async function reloadApiRenderCacheByKeyAndValue(key, cacheItem) {
    const apiK = getApiKeyByCacheKey(key);
    const res = await apiRenderRequest(cacheItem.api, apiK);
    // 缓存
    setApiRenderCache(res, cacheItem.api, apiK);
}
/**
 * 重新加载 api 缓存
 * @param apiKeys 要加载的 api 缓存的 key，可以是具名函数，或者唯一ID，不能是匿名函数，不指定时则重新加载全部
 */
async function reloadApiRenderCache(...apiKeys) {
    if (!apiKeys) {
        // 如果没有指定 api 则重新加载全部
        for (const [key, value] of apiRenderMap.entries()) {
            if (!value || !value.api) {
                apiRenderMap.delete(key);
                continue;
            }
            await reloadApiRenderCacheByKeyAndValue(key, value);
        }
        return Promise.resolve();
    }
    for (let apiKey of apiKeys) {
        // 重新加载单个缓存
        const cache = apiRenderMap.get(apiKey);
        if (!cache || !cache.api) {
            return Promise.resolve();
        }
        await reloadApiRenderCacheByKeyAndValue(apiKey, cache);
    }
    // 清理
    clearTimeoutCache();
}

/**
 * 数组工具
 */
class ArrayUtil {
    /**
     * 将数组循环拼接
     * @param arr 数组
     * @param count 循环次数
     */
    repeat(arr, count) {
        return arr.join('').repeat(count);
    }
    /**
     * 获取数组的最后一个元素
     * @param arr 数组
     * @param defaultItem 如果为空时，返回的默认值，非必传
     */
    finalItem(arr, defaultItem) {
        if (ObjectUtil$1.isEmpty(arr)) {
            return defaultItem;
        }
        return arr[arr.length - 1] || defaultItem;
    }
    /**
     * 向数组中添加新的元素并返回，如果数组为空，则返回包含新元素的数组
     * @param arr 数组
     * @param item 新元素
     */
    push(arr, item) {
        if (Array.isArray(arr) && arr.length >= 0) {
            arr.push(item);
            return arr;
        }
        return [item];
    }
    /**
     * 判断数组是否为空数组
     * @param array 数组
     */
    isEmpty(array) {
        return !array || array.length === 0;
    }
    /**
     * 判断数组为非空数组
     * @param array 数组
     */
    isNotEmpty(array) {
        return !this.isEmpty(array);
    }
    /**
     * 如果数组为空则替换，否则直接返回
     * @param array 判空的数组
     * @param defaultArray 替换的数组
     */
    defaultIfEmpty(array, defaultArray) {
        return this.isEmpty(array) ? defaultArray : array;
    }
    /**
     * 固定长度首部添加元素，返回删除的元素（向数组开头添加元素，如果长度超出指定长度，则删除尾部元素）
     * @param array 数组
     * @param len 固定的长度
     * @param items 要添加的元素
     */
    fixedUnshift(array, len, ...items) {
        array.unshift(...items.reverse());
        let deleteArr = [];
        if (array.length > len) {
            deleteArr = array.splice(len, array.length - len);
        }
        return deleteArr;
    }
    /**
     * 固定长度尾部添加元素，返回删除的元素（向数组最后添加元素，如果长度超出指定长度，则删除首部元素）
     * @param array 数组
     * @param len 固定的长度
     * @param items 要添加的元素
     */
    fixedPush(array, len, ...items) {
        array.push(...items);
        let deleteArr = [];
        if (array.length > len) {
            deleteArr = array.splice(0, array.length - len);
        }
        return deleteArr;
    }
    /**
     * 深拷贝
     * @param array 数组
     */
    deepCopy(array) {
        if (!Array.isArray(array)) {
            return [];
        }
        return array.map(item => {
            if (Array.isArray(item)) {
                return this.deepCopy(item);
            }
            else if (typeof item === 'object' && item !== null) {
                return ObjectUtil$1.deepCopy(item);
            }
            return item;
        });
    }
    /**
     * 深合并
     * @param targets 目标数组
     * @param sources 源数组
     */
    deepAssign(targets, sources) {
        if (!sources || !Array.isArray(sources)) {
            return targets;
        }
        return sources.map((item, index) => {
            if (item === undefined || item === null) {
                return targets[index];
            }
            if (Array.isArray(item)) {
                if (!!targets[index] && !Array.isArray(targets[index])) {
                    return this.deepAssign([], item);
                }
                // @ts-ignore
                return this.deepAssign(targets[index] || [], item);
            }
            else if (typeof item === 'object') {
                if (!!targets[index] && Array.isArray(targets[index])) {
                    return ObjectUtil$1.deepAssign({}, item);
                }
                return ObjectUtil$1.deepAssign(targets[index] || {}, item);
            }
            return item;
        });
    }
    /**
     * 去重
     * @param arr 要去重的数组
     * @param uniMapper 去重处理器，默认按当前元素去重
     */
    unique(arr, uniMapper) {
        const uniArr = [];
        return arr.reduce((acc, cur) => {
            const uniK = uniMapper ? uniMapper(cur) : cur;
            if (!uniArr.includes(uniK)) {
                acc.push(cur);
                uniArr.push(uniK);
            }
            return acc;
        }, []);
    }
    /**
     * 取交集
     * @param arrs 要取交集的数组集
     */
    intersection(...arrs) {
        const [first, ...rest] = arrs;
        return this.unique(first.filter((item) => rest.every(array => array.includes(item))));
    }
    /**
     * 取并集
     * @param arrs 要取并集的数组集
     */
    union(...arrs) {
        return this.unique([].concat(...arrs));
    }
    /**
     * 取差集
     * @param arrs 要取差集的数组集
     */
    difference(...arrs) {
        const arrf = this.intersection(...arrs);
        return [].concat(...arrs).reduce((acc, curr) => {
            if (!acc.includes(curr) && !arrf.includes(curr)) {
                acc.push(curr);
            }
            return acc;
        }, []);
    }
    /**
     * 分组
     * @param arr 数组
     * @param keyMapper 分组关键字处理器
     */
    groupBy(arr, keyMapper) {
        return arr.reduce((res, item) => {
            const key = keyMapper(item).toString();
            if (!res[key]) {
                res[key] = [];
            }
            res[key].push(item);
            return res;
        }, {});
    }
}
var ArrayUtil$1 = new ArrayUtil();

/**
 * 对象工具
 */
class ObjectUtil {
    /**
     * 判断任意一种类型数据是否为空：字符串、null、undefined、数组、对象等
     * @param value 数据
     */
    isEmpty(value) {
        // 如果值未定义或者为null，返回true
        if (value == null) {
            return true;
        }
        // 如果值是字符串，并且长度为0，返回true
        if (typeof value === 'string' && value.trim() === '') {
            return true;
        }
        // 如果值是数组，并且长度为0，返回true
        if (Array.isArray(value) && value.length === 0) {
            return true;
        }
        // 如果值是对象，并且没有可枚举的属性，返回true
        if (typeof value === 'object' && Object.keys(value).length === 0) {
            return true;
        }
        // 其他情况，返回false
        return false;
    }
    /**
     * 深拷贝
     * @param obj 对象
     */
    deepCopy(obj) {
        if (typeof obj !== 'object' || obj === null) {
            return obj;
        }
        const copy = Array.isArray(obj) ? [] : {};
        for (const key in obj) {
            if (obj.hasOwnProperty(key)) {
                const value = obj[key];
                // @ts-ignore
                copy[key] = Array.isArray(value) ? ArrayUtil$1.deepCopy(obj[key]) : this.deepCopy(value);
            }
        }
        return copy;
    }
    /**
     * 深合并，对于 source 中没有的属性不会覆盖
     * @param target 目标对象
     * @param sources 源对象
     */
    deepAssign(target, ...sources) {
        if (!sources) {
            return target;
        }
        for (let source of sources) {
            if (typeof source !== "object") {
                continue;
            }
            for (const key in source) {
                if (source.hasOwnProperty(key)) {
                    const value = source[key];
                    if (value === undefined || value === null)
                        continue;
                    if (Array.isArray(value)) {
                        if (!!value && !Array.isArray(value)) {
                            // @ts-ignore
                            target[key] = ArrayUtil$1.deepAssign([], source[key]);
                        }
                        else {
                            // @ts-ignore
                            target[key] = ArrayUtil$1.deepAssign(target[key] || [], source[key]);
                        }
                    }
                    else {
                        if (typeof value !== 'object') {
                            target[key] = value;
                        }
                        else {
                            if (!!target[key] && Array.isArray(target[key])) {
                                // @ts-ignore
                                target[key] = this.deepAssign({}, value);
                            }
                            else {
                                // @ts-ignore
                                target[key] = this.deepAssign(target[key] || {}, value);
                            }
                        }
                    }
                }
            }
        }
        return target;
    }
    /**
     * 深合并，target 不可为空，target[targetKey] 可为空，默认赋值为 {}
     * @param target 目标对象
     * @param targetKey 目标对象 key
     * @param sources 源对象
     */
    deepAssignByKey(target, targetKey, ...sources) {
        if (!sources) {
            return target;
        }
        if (!target[targetKey]) {
            // @ts-ignore
            target[targetKey] = {};
        }
        // @ts-ignore
        return this.deepAssign(target[targetKey], ...sources);
    }
    /**
     * 当对象不为空时追加对应的值
     * @param obj 对象
     * @param appends 追加的值
     */
    appendIfNotEmpty(obj, appends) {
        if (!this.isEmpty(obj)) {
            return obj.toString().concat(appends);
        }
        return '';
    }
    /**
     * 对象不是布尔是如果未空则返回默认值
     * @param obj 对象
     * @param defaultValue 默认值，默认为空字符串
     */
    defaultIfNotBoolean(obj, defaultValue = '') {
        if (typeof obj === 'boolean') {
            return obj;
        }
        return this.isEmpty(obj) ? defaultValue : obj;
    }
}
var ObjectUtil$1 = new ObjectUtil();

class ApiRenderTool {
    /**
     * 匹配数组中对应值相等的项，如果是对象，则返回 valueKey 对应的值
     * @param arr 数组
     * @param valueKey value 关键字
     * @param value 匹配的 value 值
     */
    getItemByValue(arr, value, valueKey) {
        valueKey = valueKey || apiRenderConfig.defaultValueKey;
        if (Array.isArray(arr)) {
            return arr.find((item) => item && (item[valueKey] === value));
        }
        return arr[valueKey];
    }
    /**
     * 通过 value 匹配获取 pai 数组数据中对应的项，如果数据为对象，则返回 valueKey 对应的值
     * @param api 请求的 api 函数
     * @param valueKey value关键字
     * @param value 匹配的 value 值
     * @param apiKey api 的 key 值，可用来声明 api 缓存的key，api 请求的数据会自动缓存，所以 api 函数尽量使用有名函数，或者如果使用匿名函数，在不保证匿名函数会字符会重复的前提下，请提供该字段
     */
    async getApiItem(api, valueKey, value, apiKey) {
        const res = await getApiRenderCache(api, apiKey);
        if (res) {
            return this.getItemByValue(res, value, valueKey);
        }
        return undefined;
    }
    /**
     * 解析项，判断项类型，如果项是对象，则返回 label 字段值，否则直接返回，如果项为空，则返回空字符串，未
     * @param item
     * @param labelKey
     */
    renderValueByItem(item, labelKey) {
        if (ObjectUtil$1.isEmpty(item))
            return '';
        if (typeof item === 'object') {
            const value = item[labelKey || apiRenderConfig.defaultLabelKey];
            return ObjectUtil$1.defaultIfNotBoolean(value);
        }
        else {
            return ObjectUtil$1.defaultIfNotBoolean(item);
        }
    }
    /**
     * 解析 api 获取到的数据，通过 valueKey 和 value 匹配对应的项，然后返回该项的 labelKey 字段对应的值，默认返回空字符串
     * <p>这里存在两种情况</p>
     * - api 返回的数据是数组，则通过遍历数组找到 valueKey 对应值为 value 的项，然后返回 labelKey 对应的值
     * - api 返回的数据是对象，则找到 valueKey 字段对应的值作为项，如果项是一个对象，则返回 labelKey 对应的值，否则直接返回
     * @param api 请求的 api 函数
     * @param valueKey value关键字
     * @param value 匹配的 value 值
     * @param labelKey 最终解析返回的字段
     * @param apiKey api 的 key 值，可用来声明 api 缓存的key，api 请求的数据会自动缓存，所以 api 函数尽量使用有名函数，或者如果使用匿名函数，在不保证匿名函数会字符会重复的前提下，请提供该字段
     */
    async renderValue(api, valueKey, value, labelKey, apiKey) {
        const item = await this.getApiItem(api, valueKey, value, apiKey);
        this.renderValueByItem(item, labelKey);
    }
    /**
     * 解析 api 获取到的数据（只能是数组）为选项，如 select、radio 等组件需要的数据
     * <p>生成如下的数据结构</p>
     * ```js
     * [{
     *  label: '',
     *  value: '',
     *  key: ''
     * }]
     * ```
     * @param api 请求的 api 函数
     * @param valueKey value关键字
     * @param labelKey 最终解析返回的字段
     * @param apiKey api 的 key 值，可用来声明 api 缓存的key，api 请求的数据会自动缓存，所以 api 函数尽量使用有名函数，或者如果使用匿名函数，在不保证匿名函数会字符会重复的前提下，请提供该字段
     */
    async renderOptions(api, valueKey, labelKey, apiKey) {
        const list = await getApiRenderCache(api, apiKey);
        if (!list)
            return [];
        return list.map(item => {
            return {
                label: item[labelKey || apiRenderConfig.defaultLabelKey],
                value: item[valueKey || apiRenderConfig.defaultValueKey],
                key: item[valueKey || apiRenderConfig.defaultValueKey],
            };
        });
    }
}
var ApiRenderUtil = new ApiRenderTool();

async function renderApiValueByOptions(apiOptions, apiKey, value, valueKey, labelKey) {
    await getApiDataIfNot(apiOptions, apiKey);
    const res = apiMapRef.value[apiKey];
    if (!res)
        return '';
    labelKey = getLabelKey(apiKey, labelKey);
    valueKey = getValueKey(apiKey, valueKey);
    const item = ApiRenderUtil.getItemByValue(res, value, valueKey);
    return ApiRenderUtil.renderValueByItem(item, labelKey);
}

class TreeUtil {
    /**
     * 默认的级别属性名
     */
    LEVEL_NAME = '__tree_level';
    /**
     * 默认的开始级别
     */
    LEVEL_BEGIN = 1;
    /**
     * 已构建标记
     */
    BUILT_TAG = '__built_tag';
    /**
     * 构建树结构（保留非根节点）
     * @param list 数组
     * @param idName 主键属性名
     * @param parentName 父级主键属性名
     * @param childName 子节点属性名
     * @param levelName 级别属性名
     * @param mapper 节点处理函数
     * @param leafMapper 叶子节点处理函数
     */
    buildTree(list, idName, parentName, childName, levelName, mapper, leafMapper) {
        if (!list)
            return [];
        let newList = ArrayUtil$1.deepCopy(list);
        levelName = levelName || this.LEVEL_NAME;
        this.buildTreeMapper(newList, idName, parentName, (current, parent, parentChildList, level) => {
            if (level !== this.LEVEL_BEGIN) {
                // 根节点遍历时没有父级
                parent[childName] = parentChildList;
            }
            current[levelName] = level;
            mapper && mapper(current, parent, parentChildList, level);
            return current;
        }, leafMapper);
        return newList;
    }
    /**
     * 构建树结构
     * @param list 数组
     * @param idName 主键属性名
     * @param parentName 父级主键属性名
     * @param childName 子节点属性名
     * @param levelName 级别属性名
     * @param mapper 节点处理函数
     * @param leafMapper 叶子节点处理函数
     */
    buildCommonTree(list, idName, parentName, childName, levelName, mapper, leafMapper) {
        return this.filterRoot(this.buildTree(list, idName, parentName, childName, levelName, mapper, leafMapper));
    }
    /**
     * 获取根节点数组（用于构建树结构后清楚非根节点数据）
     * @param list 数组
     * @param levelName 级别属性名
     */
    filterRoot(list, levelName) {
        levelName = levelName || this.LEVEL_NAME;
        return list.filter(item => item[levelName] === this.LEVEL_BEGIN);
    }
    /**
     * 递归树结构变换器
     * @param list 全数组
     * @param idName 主键属性名
     * @param parentName 父级主键属性名
     * @param mapper 节点处理函数
     * @param leafMapper 叶子节点处理函数
     */
    buildTreeMapper(list, idName, parentName, mapper, leafMapper) {
        if (ArrayUtil$1.isEmpty(list))
            return;
        for (let item of list) {
            const level = this.LEVEL_BEGIN;
            if (item[this.BUILT_TAG])
                return;
            mapper && mapper(item, null, [], level);
            this.toTreeMapper(list, item, idName, parentName, mapper, leafMapper, level);
        }
    }
    /**
     * 递归树结构变换器
     * @param list 全数组
     * @param obj 父级节点
     * @param idName 主键属性名
     * @param parentName 父级主键属性名
     * @param mapper 节点处理函数
     * @param leafMapper 叶子节点处理函数
     * @param parentLevel 上级层级
     */
    toTreeMapper(list, obj, idName, parentName, mapper, leafMapper, parentLevel) {
        const childList = [];
        for (let item of list) {
            if (item[parentName] === obj[idName]) {
                item[this.BUILT_TAG] = true;
                childList.push(item);
            }
        }
        if (childList.length > 0) {
            for (let children of childList) {
                const level = (!parentLevel ? this.LEVEL_BEGIN - 1 : parentLevel) + 1;
                const child = mapper ? mapper(children, obj, childList, level) : children;
                this.toTreeMapper(list, child, idName, parentName, mapper, leafMapper, level);
            }
        }
        else {
            leafMapper && leafMapper(obj);
        }
    }
}
var TreeUtil$1 = new TreeUtil();

function renderApiTreeByOptions(apiOptions, apiKey, options) {
    getApiDataIfNot(apiOptions, apiKey);
    const res = getApiMapRefValue(apiKey);
    if (!res || !Array.isArray(res))
        return [];
    let { labelKey, valueKey, treeLabelKey, treeValueKey, treeChildrenKey, parentKey, idKey } = options;
    labelKey = getLabelKey(apiKey, labelKey);
    valueKey = getValueKey(apiKey, valueKey);
    treeLabelKey = getTreeLabelKey(apiKey, treeLabelKey);
    treeValueKey = getValueKey(apiKey, treeValueKey);
    treeChildrenKey = getTreeChildrenKey(apiKey, treeChildrenKey);
    parentKey = getTreeParentKey(apiKey, parentKey);
    idKey = getTreeIdKey(apiKey, idKey);
    const list = res.map(item => {
        return {
            [treeLabelKey]: item[labelKey],
            [treeValueKey]: item[valueKey],
        };
    });
    return TreeUtil$1.buildCommonTree(list, idKey, parentKey, treeChildrenKey);
}

function renderApiOptionsByOptions(apiOptions, apiKey, options) {
    getApiDataIfNot(apiOptions, apiKey);
    const res = getApiMapRefValue(apiKey);
    if (!res || !Array.isArray(res))
        return [];
    let { valueKey, labelKey, dataValueKey, dataLabelKey } = options;
    labelKey = getLabelKey(apiKey, labelKey);
    valueKey = getValueKey(apiKey, valueKey);
    dataValueKey = getOptionsValueKey(apiKey, dataValueKey);
    dataLabelKey = getOptionsLabelKey(apiKey, dataLabelKey);
    return res.map(item => {
        return {
            [dataLabelKey]: item[labelKey],
            [dataValueKey]: item[valueKey],
            key: item[valueKey]
        };
    });
}

const apiOptions = {};
// api 响应式数据缓存
const apiMapRef = ref({});
// 关键字映射
const keyMap = {};
/**
 * 获取 api 响应式数据缓存值
 * @param apiKey api 数据缓存 key
 */
function getApiMapRefValue(apiKey) {
    return apiMapRef.value[apiKey];
}
/**
 * 获取默认的项配置
 */
function getDefaultApiRenderOptionsConfig() {
    return {
        labelKey: apiRenderConfig.defaultLabelKey,
        valueKey: apiRenderConfig.defaultValueKey,
        treeValueKey: apiRenderConfig.defaultTreeValueKey,
        treeLabelKey: apiRenderConfig.defaultTreeLabelKey,
        treeIdKey: apiRenderConfig.defaultTreeIdKey,
        treeChildrenKey: apiRenderConfig.defaultTreeChildrenKey,
        treeParentIdKey: apiRenderConfig.defaultTreeParentIdKey,
        optionsValueKey: apiRenderConfig.defaultOptionsValueKey,
        optionsLabelKey: apiRenderConfig.defaultOptionsLabelKey,
    };
}
/**
 * 设置 api 项
 */
function defineApiRenderOptions(ops) {
    const keysMap = {};
    // 赋值默认值
    const defaultApiRenderOptionsConfig = getDefaultApiRenderOptionsConfig();
    for (let opsKey in ops) {
        ops[opsKey] = Object.assign({}, defaultApiRenderOptionsConfig, ops[opsKey]);
        keysMap[opsKey] = opsKey;
    }
    Object.assign(apiOptions, ops);
    function renderApiValue(apiKey, value, valueKey, labelKey) {
        return renderApiValueByOptions(ops, apiKey, value, valueKey, labelKey);
    }
    function renderApiOptions(apiKey, options) {
        return renderApiOptionsByOptions(ops, apiKey, options);
    }
    function renderApiTree(apiKey, options) {
        return renderApiTreeByOptions(ops, apiKey, options);
    }
    function reloadApiRenderOptionsData(...apiKeys) {
        const keys = apiKeys ? apiKeys.map(item => item.toString()) : Object.keys(apiMapRef.value);
        reloadApiRenderCache(...keys).then(() => {
            for (let key of keys) {
                getApiData(ops, key);
            }
        });
    }
    const ApiRender = defineComponent({
        name: 'ApiRender',
        props: {
            apiKey: {
                type: [String],
                required: true
            },
            value: {
                type: [String, Number, Boolean, Object]
            }
        },
        setup(props) {
            const renderValue = ref('');
            const { apiKey, value } = props;
            // @ts-ignore
            renderApiValueByOptions(ops, apiKey, value).then(res => {
                renderValue.value = res;
            });
            return () => renderValue.value;
        }
    });
    return {
        keysMap,
        ApiRender,
        /**
         * 解析 api 数据，匹配值，返回 label
         * @param apiKey api 的 option 关键字
         * @param value 要匹配的 value 值
         * @param valueKey 要匹配的数据 value 关键字
         * @param labelKey 返回的 label 关键字
         */
        renderApiValue,
        /**
         * 解析 api 数据为选择项数据
         * @param apiKey api 的 option 关键字
         * @param options 配置项
         */
        renderApiOptions,
        /**
         * 解析 api 数据为树结构数据
         * @param apiKey api 的 option 关键字
         * @param options 配置项
         */
        renderApiTree,
        /**
         * 重新加载 api 项数据
         * @param apiKey 要重载的 api 项 key
         */
        reloadApiRenderOptionsData
    };
}
/**
 * 获取 api 数据并放到 api 响应式缓存中
 * @param apiOptions api 项
 * @param apiKey api 缓存 key
 */
async function getApiData(apiOptions, apiKey) {
    const api = apiOptions[apiKey];
    if (api) {
        keyMap[apiKey] = getDefaultApiRenderOptionsConfig();
        if (api instanceof Function) {
            apiMapRef.value[apiKey] = await getApiRenderCache(api, apiKey.toString());
        }
        else {
            apiMapRef.value[apiKey] = await getApiRenderCache(api.api, apiKey.toString());
            keyMap[apiKey] = Object.assign({}, keyMap[apiKey], {
                labelKey: api.labelKey || apiRenderConfig.defaultLabelKey,
                valueKey: api.valueKey || apiRenderConfig.defaultValueKey
            });
        }
    }
}
/**
 * 先判断有没有缓存再获取 api 数据并放到 api 响应式缓存中
 * @param apiOptions api 项
 * @param apiKey api 缓存 key
 */
async function getApiDataIfNot(apiOptions, apiKey) {
    const apiValue = apiMapRef.value[apiKey];
    if (!apiValue) {
        await getApiData(apiOptions, apiKey);
    }
}
/**
 * 获取 api 的 label 关键字，如果存在 labelKey 则返回 labelKey
 * @param apiKey api 缓存 key
 * @param labelKey 覆盖的 label 关键字
 */
function getLabelKey(apiKey, labelKey) {
    if (labelKey)
        return labelKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].labelKey || apiRenderConfig.defaultLabelKey;
    return apiRenderConfig.defaultLabelKey;
}
/**
 * 获取 api 的 value 关键字，如果存在 valueKey 则返回 valueKey
 * @param apiKey api 缓存 key
 * @param valueKey 覆盖的 value 关键字
 */
function getValueKey(apiKey, valueKey) {
    if (valueKey)
        return valueKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].valueKey || apiRenderConfig.defaultValueKey;
    return apiRenderConfig.defaultValueKey;
}
function getTreeLabelKey(apiKey, treeLabelKey) {
    if (treeLabelKey)
        return treeLabelKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].treeLabelKey || apiRenderConfig.defaultTreeLabelKey;
    return apiRenderConfig.defaultTreeLabelKey;
}
function getTreeIdKey(apiKey, treeIdKey) {
    if (treeIdKey)
        return treeIdKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].treeIdKey || apiRenderConfig.defaultTreeIdKey;
    return apiRenderConfig.defaultTreeIdKey;
}
function getTreeParentKey(apiKey, treeParentKey) {
    if (treeParentKey)
        return treeParentKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].treeParentIdKey || apiRenderConfig.defaultTreeParentIdKey;
    return apiRenderConfig.defaultTreeParentIdKey;
}
function getTreeChildrenKey(apiKey, treeChildrenKey) {
    if (treeChildrenKey)
        return treeChildrenKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].treeChildrenKey || apiRenderConfig.defaultTreeChildrenKey;
    return apiRenderConfig.defaultTreeChildrenKey;
}
function getOptionsValueKey(apiKey, optionsValueKey) {
    if (optionsValueKey)
        return optionsValueKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].optionsValueKey || apiRenderConfig.defaultOptionsValueKey;
    return apiRenderConfig.defaultOptionsValueKey;
}
function getOptionsLabelKey(apiKey, optionsLabelKey) {
    if (optionsLabelKey)
        return optionsLabelKey;
    if (keyMap[apiKey])
        return keyMap[apiKey].optionsLabelKey || apiRenderConfig.defaultOptionsLabelKey;
    return apiRenderConfig.defaultOptionsLabelKey;
}

const ApiRenderUtils = {
    ArrayUtil: ArrayUtil$1,
    ObjectUtil: ObjectUtil$1,
    TreeUtil: TreeUtil$1
};
const apiRenderVue = apiRenderVue$1;

export { ApiRenderUtil as ApiRenderTool, ApiRenderUtils, apiRenderVue as default, defineApiRenderOptions, getApiRenderCache, getApiRenderConfig, reloadApiRenderCache, setApiRenderCache, setApiRenderConfig };
